@@ -28,6 +28,8 @@ pango_attr_type_get_type (void)
       { PANGO_ATTR_SCALE, "PANGO_ATTR_SCALE", "scale" },
       { PANGO_ATTR_FALLBACK, "PANGO_ATTR_FALLBACK", "fallback" },
       { PANGO_ATTR_LETTER_SPACING, "PANGO_ATTR_LETTER_SPACING", "letter-spacing" },
+      { PANGO_ATTR_UNDERLINE_COLOR, "PANGO_ATTR_UNDERLINE_COLOR", "underline-color" },
+      { PANGO_ATTR_STRIKETHROUGH_COLOR, "PANGO_ATTR_STRIKETHROUGH_COLOR", "strikethrough-color" },
       { 0, NULL, NULL }
     };
     etype = g_enum_register_static ("PangoAttrType", values);
@@ -217,6 +219,25 @@ pango_ellipsize_mode_get_type (void)
 }
 
 
+/* enumerations from "pango-renderer.h" */
+GType
+pango_render_part_get_type (void)
+{
+  static GType etype = 0;
+  if (etype == 0) {
+    static const GEnumValue values[] = {
+      { PANGO_RENDER_PART_FOREGROUND, "PANGO_RENDER_PART_FOREGROUND", "foreground" },
+      { PANGO_RENDER_PART_BACKGROUND, "PANGO_RENDER_PART_BACKGROUND", "background" },
+      { PANGO_RENDER_PART_UNDERLINE, "PANGO_RENDER_PART_UNDERLINE", "underline" },
+      { PANGO_RENDER_PART_STRIKETHROUGH, "PANGO_RENDER_PART_STRIKETHROUGH", "strikethrough" },
+      { 0, NULL, NULL }
+    };
+    etype = g_enum_register_static ("PangoRenderPart", values);
+  }
+  return etype;
+}
+
+
 /* enumerations from "pango-script.h" */
 GType
 pango_script_get_type (void)
