@@ -52,10 +52,6 @@ struct _PangoWin32FontClass
   PangoFontClass parent_class;
 };
 
-static PangoFontClass *parent_class;	/* Parent class structure for PangoWin32Font */
-
-static void pango_win32_font_class_init (PangoWin32FontClass *class);
-static void pango_win32_font_init       (PangoWin32Font      *win32font);
 static void pango_win32_font_dispose    (GObject             *object);
 static void pango_win32_font_finalize   (GObject             *object);
 
@@ -142,33 +138,7 @@ pango_win32_get_context (void)
   return result;
 }
 
-static GType
-pango_win32_font_get_type (void)
-{
-  static GType object_type = 0;
-
-  if (!object_type)
-    {
-      static const GTypeInfo object_info =
-      {
-        sizeof (PangoWin32FontClass),
-        (GBaseInitFunc) NULL,
-        (GBaseFinalizeFunc) NULL,
-        (GClassInitFunc) pango_win32_font_class_init,
-        NULL,           /* class_finalize */
-        NULL,           /* class_data */
-        sizeof (PangoWin32Font),
-        0,              /* n_preallocs */
-        (GInstanceInitFunc) pango_win32_font_init,
-      };
-      
-      object_type = g_type_register_static (PANGO_TYPE_FONT,
-                                            "PangoWin32Font",
-                                            &object_info, 0);
-    }
-  
-  return object_type;
-}
+G_DEFINE_TYPE (PangoWin32Font, pango_win32_font, PANGO_TYPE_FONT)
 
 static void 
 pango_win32_font_init (PangoWin32Font *win32font)
@@ -231,8 +201,6 @@ pango_win32_font_class_init (PangoWin32FontClass *class)
   GObjectClass *object_class = G_OBJECT_CLASS (class);
   PangoFontClass *font_class = PANGO_FONT_CLASS (class);
 
-  parent_class = g_type_class_peek_parent (class);
-  
   object_class->finalize = pango_win32_font_finalize;
   object_class->dispose = pango_win32_font_dispose;
   
@@ -428,6 +396,41 @@ pango_win32_render (HDC               hdc,
   g_free (dX);
 }
 
+void 
+pango_win32_render_transformed (HDC         hdc,
+			      const PangoMatrix *matrix,
+			      PangoFont         *font,
+			      PangoGlyphString  *glyphs,
+			      int                x, 
+			      int                y)
+{
+  XFORM xForm;
+  XFORM xFormPrev = {1.0, 0.0, 0.0, 1.0, 0.0, 0.0};
+  int   mode = GetGraphicsMode (hdc);
+
+  if (!SetGraphicsMode (hdc, GM_ADVANCED))
+    g_warning ("SetGraphicsMode() failed");
+  else if (!GetWorldTransform (hdc, &xFormPrev))
+    g_warning ("GetWorldTransform() failed");
+  else if (matrix)
+    {
+      xForm.eM11 = matrix->xx;
+      xForm.eM12 = matrix->xy;
+      xForm.eM21 = matrix->yx;
+      xForm.eM22 = matrix->yy;
+      xForm.eDx = matrix->x0;
+      xForm.eDy = matrix->y0;
+      if (!SetWorldTransform (hdc, &xForm))
+        g_warning ("GetWorldTransform() failed");
+    }
+
+  pango_win32_render (hdc, font, glyphs, x/PANGO_SCALE, y/PANGO_SCALE);
+
+  /* restore */
+  SetWorldTransform (hdc, &xFormPrev);
+  SetGraphicsMode (hdc, mode);
+}
+
 static void
 pango_win32_font_get_glyph_extents (PangoFont      *font,
 				    PangoGlyph      glyph,
@@ -617,7 +620,7 @@ pango_win32_font_dispose (GObject *object)
   if (!win32font->in_cache && win32font->fontmap)
     pango_win32_fontmap_cache_add (win32font->fontmap, win32font);
 
-  G_OBJECT_CLASS (parent_class)->dispose (object);
+  G_OBJECT_CLASS (pango_win32_font_parent_class)->dispose (object);
 }
 
 static void
@@ -636,7 +639,7 @@ pango_win32_font_finalize (GObject *object)
  
   g_object_unref (win32font->fontmap);
 
-  G_OBJECT_CLASS (parent_class)->finalize (object);
+  G_OBJECT_CLASS (pango_win32_font_parent_class)->finalize (object);
 }
 
 static PangoFontDescription *
@@ -801,6 +804,29 @@ pango_win32_render_layout_line (HDC              hdc,
 	  points[0].y = points[1].y = y + 2;
 	  Polyline (hdc, points, 2);
 	  break;
+	case PANGO_UNDERLINE_ERROR:
+          {
+            int point_x;
+            int counter = 0;
+	    int end_x = x + PANGO_PIXELS (x_off + ink_rect.x + ink_rect.width);
+
+            for (point_x = x + PANGO_PIXELS (x_off + ink_rect.x) - 1;
+                 point_x <= end_x;
+                 point_x += 2)
+            {
+	      points[0].x = point_x;
+	      points[1].x = MAX (point_x + 1, end_x);
+
+              if (counter)
+	        points[0].y = points[1].y = y + 2;
+              else
+	        points[0].y = points[1].y = y + 3;
+
+	      Polyline (hdc, points, 2);
+              counter = (counter + 1) % 2;
+            }
+          }
+	  break;
 	case PANGO_UNDERLINE_LOW:
 	  points[0].x = x + PANGO_PIXELS (x_off + ink_rect.x) - 1;
 	  points[0].y = points[1].y = y + PANGO_PIXELS (ink_rect.y + ink_rect.height) + 2;
